﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Airbus
{
    //класс для хранения коллекции карт
    internal class MapsCollection
    {
        //словарь (хранилище) с картами
        readonly Dictionary<string, MapWithSetPlanesGeneric<IDrawningObject, AbstractMap>> _mapStorage;

        //возвращение списка названий карт
        public List<string> Keys => _mapStorage.Keys.ToList();

        //разделитель для записи информации по элементу словаря в файл
        private readonly char separatorDict = '|';

        //разделитель для записи коллекции данных в файл
        private readonly char separatorData = ';';

        //ширина окна отрисовки
        private readonly int _pictureWidth;

        //высота окна отрисовки
        private readonly int _pictureHeight;

        //конструктор
        public MapsCollection(int pictureWidth, int pictureHeight)
        {
            _mapStorage = new Dictionary<string, MapWithSetPlanesGeneric<IDrawningObject, AbstractMap>>();
            _pictureWidth = pictureWidth;
            _pictureHeight = pictureHeight;
        }

        //добавление карты
        public void AddMap(string name, AbstractMap map)
        {
            var NewElem = new MapWithSetPlanesGeneric<IDrawningObject, AbstractMap>(
            _pictureWidth, _pictureHeight, map);
            _mapStorage.Add(name, NewElem);
        }

        //удаление карты
        public void DelMap(string name)
        {
            _mapStorage.Remove(name);
        }

        // Метод записи информации в файл
        private static void WriteToFile(string text, FileStream stream)
        {
            byte[] info = new UTF8Encoding(true).GetBytes(text);
            stream.Write(info, 0, info.Length);
        }

        // Сохранение информации по автомобилям в хранилище в файл
        public bool SaveData(string filename)
        {
            if (File.Exists(filename))
            {
                File.Delete(filename);
            }

            using (FileStream fs = new(filename, FileMode.Create))
            {
                WriteToFile($"MapsCollection{Environment.NewLine}", fs);
                foreach (var storage in _mapStorage)
                {

                    WriteToFile($"{storage.Key}{separatorDict}{storage.Value.GetData(separatorDict, separatorData)}" +
                        $"{Environment.NewLine}", fs);
                }
            }

            return true;
        }

        // Загрузка нформации по автомобилям на парковках из файла
        public bool LoadData(string filename)
        {
            if (!File.Exists(filename))
            {
                return false;
            }

            string bufferTextFromFile = "";

            using (FileStream fs = new(filename, FileMode.Open))
            {
                byte[] b = new byte[fs.Length];
                UTF8Encoding temp = new(true);

                while (fs.Read(b, 0, b.Length) > 0)
                {
                    bufferTextFromFile += temp.GetString(b);
                }
            }

            var strs = bufferTextFromFile.Split(new char[] { '\n', '\r' }, StringSplitOptions.RemoveEmptyEntries);

            if (!strs[0].Contains("MapsCollection"))
            {
                //если нет такой записи, то это не те данные
                return false;
            }

            //очищаем записи
            _mapStorage.Clear();

            for (int i = 1; i < strs.Length; ++i)
            {
                var elem = strs[i].Split(separatorDict);
                AbstractMap map = null;

                switch (elem[1])
                {
                    case "SimpleMap":
                        map = new SimpleMap();
                        break;
                    case "DesertStormMap":
                        map = new DesertStormMap();
                        break;
                    case "StarWarsMap":
                        map = new StarWarsMap();
                        break;
                }

                _mapStorage.Add(elem[0], new MapWithSetPlanesGeneric<IDrawningObject, AbstractMap>(_pictureWidth, _pictureHeight,
               map));

                _mapStorage[elem[0]].LoadData(elem[2].Split(separatorData, StringSplitOptions.RemoveEmptyEntries));
            }

            return true;
        }


        //Доступ к аэродрому
        public MapWithSetPlanesGeneric<IDrawningObject, AbstractMap> this[string ind]
        {
            get
            {
                return _mapStorage[ind];
            }
        }
    }
}
